package cn.iocoder.yudao.module.infra.service.minio;

import cn.iocoder.yudao.module.infra.config.MinioConfigProperties;
import io.minio.BucketExistsArgs;
import io.minio.MakeBucketArgs;
import io.minio.MinioClient;
import io.minio.PutObjectArgs;
import io.minio.RemoveObjectArgs;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

/**
 * MinIO 文件服务类
 *
 * @author AI Assistant
 */
@Slf4j
@Service
public class MinioService {

    @Resource
    private MinioConfigProperties minioConfigProperties;

    private MinioClient minioClient;

    /**
     * 初始化 MinIO 客户端
     */
    @PostConstruct
    public void init() {
        try {
            this.minioClient = MinioClient.builder()
                    .endpoint(minioConfigProperties.getEndpoint())
                    .credentials(minioConfigProperties.getAccessKey(), minioConfigProperties.getSecretKey())
                    .build();

            // 检查桶是否存在，不存在则创建
            boolean bucketExists = minioClient.bucketExists(
                    BucketExistsArgs.builder()
                            .bucket(minioConfigProperties.getBucketName())
                            .build()
            );

            if (!bucketExists) {
                minioClient.makeBucket(
                        MakeBucketArgs.builder()
                                .bucket(minioConfigProperties.getBucketName())
                                .build()
                );
                log.info("创建存储桶: {}", minioConfigProperties.getBucketName());
            }

            log.info("MinIO 客户端初始化成功. Endpoint: {}, Bucket: {}", 
                    minioConfigProperties.getEndpoint(), minioConfigProperties.getBucketName());
        } catch (Exception e) {
            log.error("MinIO 客户端初始化失败", e);
            throw new RuntimeException("MinIO 客户端初始化失败", e);
        }
    }

    /**
     * 上传单个文件
     *
     * @param file 上传的文件
     * @param path 文件存储路径，如果为空则自动生成
     * @return 文件访问URL
     */
    public String uploadFile(MultipartFile file, String path) throws Exception {
        if (file.isEmpty()) {
            throw new IllegalArgumentException("上传文件不能为空");
        }

        // 如果路径为空，则生成随机路径
        if (path == null || path.trim().isEmpty()) {
            String originalFilename = file.getOriginalFilename();
            String extension = "";
            if (originalFilename != null && originalFilename.contains(".")) {
                extension = originalFilename.substring(originalFilename.lastIndexOf("."));
            }
            path = UUID.randomUUID().toString() + extension;
        }

        try (InputStream inputStream = file.getInputStream()) {
            // 上传文件到 MinIO
            minioClient.putObject(
                    PutObjectArgs.builder()
                            .bucket(minioConfigProperties.getBucketName())
                            .object(path)
                            .stream(inputStream, file.getSize(), -1)
                            .contentType(file.getContentType())
                            .build()
            );

            // 生成文件访问URL
            String fileUrl = minioConfigProperties.getEndpoint() + "/" + 
                           minioConfigProperties.getBucketName() + "/" + path;
            
            log.info("文件上传成功: {}", fileUrl);
            return fileUrl;
        } catch (Exception e) {
            log.error("文件上传失败: {}", path, e);
            throw new Exception("文件上传失败: " + e.getMessage(), e);
        }
    }

    /**
     * 上传多个文件
     *
     * @param files 上传的文件数组
     * @return 文件访问URL列表
     */
    public List<String> uploadFiles(MultipartFile[] files) throws Exception {
        if (files == null || files.length == 0) {
            throw new IllegalArgumentException("上传文件不能为空");
        }

        List<String> fileUrls = new ArrayList<>();
        
        for (MultipartFile file : files) {
            if (!file.isEmpty()) {
                String fileUrl = uploadFile(file, null);
                fileUrls.add(fileUrl);
            }
        }

        log.info("批量文件上传完成，共上传 {} 个文件", fileUrls.size());
        return fileUrls;
    }

    /**
     * 删除文件
     *
     * @param path 文件路径
     */
    public void deleteFile(String path) throws Exception {
        try {
            minioClient.removeObject(
                    RemoveObjectArgs.builder()
                            .bucket(minioConfigProperties.getBucketName())
                            .object(path)
                            .build()
            );
            log.info("文件删除成功: {}", path);
        } catch (Exception e) {
            log.error("文件删除失败: {}", path, e);
            throw new Exception("文件删除失败: " + e.getMessage(), e);
        }
    }

} 